@@ -4,6 +4,7 @@
 
 import datetime
 from errors import *
+from mozdevice import devicemanagerADB, DMError
 from mozprocess import ProcessHandlerMixin
 import multiprocessing
 import os
@@ -19,6 +20,7 @@ import time
 from emulator_battery import EmulatorBattery
 from emulator_geo import EmulatorGeo
 
+
 class LogcatProc(ProcessHandlerMixin):
     """Process handler for logcat which saves all output to a logfile.
     """
@@ -42,6 +44,7 @@ class Emulator(object):
                  arch="x86", emulatorBinary=None, res='480x800', sdcard=None,
                  userdata=None):
         self.port = None
+        self.dm = None
         self._emulator_launched = False
         self.proc = None
         self.marionette_port = None
@@ -319,6 +322,9 @@ waitFor(
             online, offline = self._get_adb_devices()
         self.port = int(list(online)[0])
 
+        self.dm = devicemanagerADB.DeviceManagerADB(adbPath=self.adb,
+                                                    deviceSerial='emulator-%d' % self.port)
+
     def start(self):
         self._check_for_b2g()
         self.start_adb()
@@ -347,6 +353,9 @@ waitFor(
         self.port = int(list(online - original_online)[0])
         self._emulator_launched = True
 
+        self.dm = devicemanagerADB.DeviceManagerADB(adbPath=self.adb,
+                                                    deviceSerial='emulator-%d' % self.port)
+
         # bug 802877
         time.sleep(10)
         self.geo.set_default_location()
@@ -369,15 +378,41 @@ waitFor(
         Install gecko into the emulator using adb push.  Restart b2g after the
         installation.
         """
+        # See bug 800102.  We use this particular method of installing
+        # gecko in order to avoid an adb bug in which adb will sometimes
+        # hang indefinitely while copying large files to the system
+        # partition.
+        push_attempts = 10
+
+        print 'installing gecko binaries...'
         # need to remount so we can write to /system/b2g
         self._run_adb(['remount'])
-        self._run_adb(['shell', 'stop', 'b2g'])
-        self._run_adb(['shell', 'rm', '-rf', '/system/b2g/*.so'])
-        print 'installing gecko binaries'
-        self._run_adb(['push', gecko_path, '/system/b2g'])
+        for root, dirs, files in os.walk(gecko_path):
+            for filename in files:
+                rel_path = os.path.relpath(os.path.join(root, filename), gecko_path)
+                system_b2g_file = os.path.join('/system/b2g', rel_path)
+                for retry in range(1, push_attempts+1):
+                    print 'pushing', system_b2g_file, '(attempt %s of %s)' % (retry, push_attempts)
+                    try:
+                        self.dm.pushFile(os.path.join(root, filename), system_b2g_file)
+                        break
+                    except DMError:
+                        if retry == push_attempts:
+                            raise
+
         print 'restarting B2G'
-        self._run_adb(['shell', 'start', 'b2g'])
-        self.wait_for_port()
+        self.dm.shellCheckOutput(['stop', 'b2g'])
+        # ensure the b2g process has fully stopped (bug 809437)
+        for i in range(0, 10):
+            time.sleep(1)
+            if self.dm.processExist('b2g') is None:
+                break
+        else:
+            raise TimeoutException("Timeout waiting for the b2g process to terminate")
+        self.dm.shellCheckOutput(['start', 'b2g'])
+
+        if not self.wait_for_port():
+            raise TimeoutException("Timeout waiting for marionette on port '%s'" % self.marionette_port)
         self.wait_for_system_message(marionette)
 
     def rotate_log(self, srclog, index=1):
