@@ -47,12 +47,7 @@ class MarionetteTestResult(unittest._TextTestResult):
         self.tests_passed.append(test)
 
     def getInfo(self, test):
-        if hasattr(test, 'jsFile'):
-            return os.path.basename(test.jsFile)
-        else:
-            return '%s.py:%s.%s' % (test.__class__.__module__,
-                                    test.__class__.__name__,
-                                    test._testMethodName)
+        return test.test_name
 
     def getDescription(self, test):
         doc_first_line = test.shortDescription()
@@ -67,6 +62,16 @@ class MarionetteTestResult(unittest._TextTestResult):
     def printLogs(self, test):
         for testcase in test._tests:
             if hasattr(testcase, 'loglines') and testcase.loglines:
+                # Don't dump loglines to the console if they only contain
+                # TEST-START and TEST-END.
+                skip_log = True
+                for line in testcase.loglines:
+                    str_line = ' '.join(line)
+                    if not 'TEST-END' in str_line and not 'TEST-START' in str_line:
+                        skip_log = False
+                        break
+                if skip_log:
+                    return
                 self.stream.writeln('START LOG:')
                 for line in testcase.loglines:
                     self.stream.writeln(' '.join(line))
@@ -83,12 +88,16 @@ class MarionetteTestResult(unittest._TextTestResult):
     def printErrorList(self, flavour, errors):
         for test, err in errors:
             self.stream.writeln(self.separator1)
-            self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
+            self.stream.writeln("%s: %s" % (flavour, self.getDescription(test)))
             self.stream.writeln(self.separator2)
             errlines = err.strip().split('\n')
             for line in errlines[0:-1]:
                 self.stream.writeln("%s" % line)
-            self.stream.writeln("TEST-UNEXPECTED-FAIL : %s" % errlines[-1])
+            if "TEST-UNEXPECTED-FAIL" in errlines[-1]:
+                self.stream.writeln(errlines[-1])
+            else:
+                self.stream.writeln("TEST-UNEXPECTED-FAIL | %s | %s" %
+                                    (self.getInfo(test), errlines[-1]))
 
     def stopTest(self, *args, **kwargs):
         unittest._TextTestResult.stopTest(self, *args, **kwargs)
@@ -185,7 +194,7 @@ class MarionetteTestRunner(object):
                  es_server=None, rest_server=None, logger=None,
                  testgroup="marionette", noWindow=False, logcat_dir=None,
                  xml_output=None, repeat=0, perf=False, perfserv=None,
-                 gecko_path=None, testvars=None):
+                 gecko_path=None, testvars=None, tree=None):
         self.address = address
         self.emulator = emulator
         self.emulatorBinary = emulatorBinary
@@ -212,6 +221,7 @@ class MarionetteTestRunner(object):
         self.perfserv = perfserv
         self.gecko_path = gecko_path
         self.testvars = None
+        self.tree = tree
 
         if testvars is not None:
             if not os.path.exists(testvars):
@@ -320,7 +330,7 @@ class MarionetteTestRunner(object):
             logfile = logfile)
 
         testgroup.set_primary_product(
-            tree = 'b2g',
+            tree = self.tree,
             buildtype = 'opt',
             revision = self.revision)
 
@@ -395,7 +405,7 @@ class MarionetteTestRunner(object):
         suite = unittest.TestSuite()
 
         if file_ext == '.ini':
-            testargs = { 'skip': 'false' }
+            testargs = {}
             if testtype is not None:
                 testtypes = testtype.replace('+', ' +').replace('-', ' -').split()
                 for atype in testtypes:
@@ -433,9 +443,9 @@ class MarionetteTestRunner(object):
                              id=os.getenv('BUILD_ID'),
                              test_date=int(time.time()))
 
-            manifest_tests = manifest.get(**testargs)
+            manifest_tests = manifest.active_tests(disabled=False)
 
-            for i in manifest_tests:
+            for i in manifest.get(tests=manifest_tests, **testargs):
                 self.run_test(i["path"], testtype)
                 if self.marionette.check_for_crash():
                     return
@@ -643,7 +653,10 @@ def parse_options():
                       'installed on the device or emulator')
     parser.add_option('--testvars', dest='testvars', action='store',
                       default=None,
-                     help='path to a JSON file with any test data required')
+                      help='path to a JSON file with any test data required')
+    parser.add_option('--tree', dest='tree', action='store',
+                      default='b2g',
+                      help='the tree that the revsion parameter refers to')
 
     options, tests = parser.parse_args()
 
@@ -689,6 +702,7 @@ def startTestRunner(runner_class, options, tests):
                           noWindow=options.noWindow,
                           revision=options.revision,
                           testgroup=options.testgroup,
+                          tree=options.tree,
                           autolog=options.autolog,
                           xml_output=options.xml_output,
                           repeat=options.repeat,
